#include <stdio.h>	/* standard C i/o facilities */
#include <unistd.h>	/* Unix System Calls */
#include <sys/types.h>	/* system data type definitions */
#include <sys/socket.h> /* socket specific definitions */
#include <netinet/in.h> /* INET constants and stuff */
#include <arpa/inet.h>	/* IP address conversion stuff */
#include <stdlib.h>	/* malloc */
#include <pthread.h>	/* Posix Threads stuff */
#include <unistd.h>	/* gethostname */

#include <netdb.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <termios.h>
#include <math.h>

#define DOOR_STARTUP		"00"
#define ACCESS_GRANTED		"01"
#define ACCESS_DENIED		"02"
#define DOOR_OPENED_NORMALLY	"03"
#define DOOR_CLOSED_NORMALLY	"04"
#define DOOR_STAY_OPENED	"05"
#define DOOR_OPENED_BY_FORCE	"06"

// The name and port of server
// #define SERVER_PORT	1500
// #define SERVER_NAME	"192.168.1.36"

#define FALSE 0
#define TRUE 1

#define READY 		0
#define FIRST_READ 	1
#define SECOND_READ 	2
#define THIRD_READ 	3
#define FOURTH_READ 	4
#define FIFTH_READ 	5
#define SIXTH_READ 	6

#define MAX_CARD 	500

#define LogData		"001"
#define DBUpReq		"010"
#define WrDoorIDNot	"011"

#define ReTiMonReq	"100"
#define LogDataReq	"101"
#define RelDoDBData	"110"
#define AllDoDBData	"111"
#define zeros		"0000000000"

// Doors Table
#define fldDOODoor_No			0
#define fldDOODoor_Name			1
#define fldDOORoom_No			2
#define fldDOODoor_Address		3
#define fldDOOEvent_Capacity		4
#define fldDOOThreshold_Capacity	5
#define fldDOOTwo_Readers_Present	6
#define fldDOODoor_Type			7
#define fldDOOInner_Reader_Active	8
#define fldDOOOuter_Reader_Active	9
#define fldDOOStay_Open_Duration	10
#define fldDOODoor_Locked		11
#define fldDOODoor_Forced_Sensor	12
#define fldDOOUsed_Calendar		13
#define fldDOOOnline_Log		14

// Cards Table
#define fldCARCard_Number		0
#define fldCARInternal_Number		1
#define fldCARSurname			2
#define fldCARFirst_Name		3
#define fldCARMiddle_Name		4
#define fldCARAttendance_Follow		5
#define fldCAREnterance_Group		6
#define fldCARDepartment		7
#define fldCAREnabled			8
#define fldCARTelephone_Number		9
#define fldCARRegister_DateTime		10
#define fldCARExpire_DateTime		11

// Calendar Table
#define fldCALCalendar_No		0
#define fldCALDescription		1
#define fldCALDay			2
#define fldCALOpen_Time			3
#define fldCALAction_No			4
#define fldCALType			5
#define fldCALAction_Date		6

#define MAX_MSG 255

#define BUFSIZE 100
#define MAXCLIENTS 10

//  baudrate settings are defined in <asm/termbits.h>, which is
//  included by <termios.h>
#define BAUDRATE B2400
// change this definition for the correct port
// #define MODEMDEVICE "/dev/ttyS0"
#define _POSIX_SOURCE 1 // POSIX compliant source
  int fd;
  struct termios oldtio,newtio;
int OpenStayDuration=3;

int SERVER_PORT;
char SERVER_NAME[50], MODEMDEVICE[50];

/*==========================================================*/
/* Global variables used by all the threads.
   clients is a list of flags, a 1 means the client is
   active (0 means available).

   client_sd is a list of socket descriptors, the only valid
   socket descriptors correspond to clients that are active
   (client_sd[i] is meaningless unless clients[i] is a 1).
*/

int clients[MAXCLIENTS]; 		/* Array of flags 1 means connected*/
char door_ip[MAXCLIENTS][MAX_MSG];
//int client_sd[MAXCLIENTS]; 		/* array of socket descriptors */

int pport;				/* Passive mode port number */

char packet[255];

FILE *card_file, *calendar_file;
int cn_array[MAX_CARD], cn2, aindex=0, Calendar_Amount=0;

char log_file_sent_array[1000][52];
int sacounter=0;

/* mutex used to restrict access to clients and client_sd to a single
   thread at a time
*/

pthread_mutex_t mtx = PTHREAD_MUTEX_INITIALIZER;

/* functions to grab and release the mutex
   If anything goes wrong the server is shut down
   (somewhat rudely)
*/

int InnerReaderActive=1;
int OuterReaderActive=1;
char myDoorIDstr[255];
int myOnline;
int myThresholdCapacity;
int myEventCapacity;
int myStartedUp=1;

void grablock() {
  /* grab the lock */
  if (pthread_mutex_lock(&mtx)!=0) {
    printf("error getting mutex lock\n");
    exit(1);
  }
}

void releaselock() {
  /* release the lock */
  if (pthread_mutex_unlock(&mtx)!=0) {
    printf("error releasing mutex lock\n");
    exit(1);
  }
}

/* a clientstruct is passed to each thread,
   to let it know which client it should handle
*/

typedef struct {
  int clientnum;
  int sd;
} clientstruct, *clientptr;

typedef struct {
  int status;
  int reader;
} params, *paramsptr;

struct 
{
  char timeOfDate[17], date[17], Day[2], Action_No_Str[9], Type[2];
} Calendar_Array[1000];

struct 
{
  char timeOfDate[17], date[17];
} CardExp_Array[MAX_CARD];

void * control_lock_led_buzer(void * arg) {
paramsptr me = (paramsptr) arg;
int status = me->status;
int reader = me->reader;
int x,y;
	char buf[2];
	int res;

	grablock();
	buf[1]='\0';
	if (status==1){ // Kapi acildi yeterince bekle, lock the door + yesili sondur + Turuncu yak
		sleep(OpenStayDuration);
		if (reader)
			buf[0]=20; // Ic okuyucu
		else
			buf[0]=16; // Dis okuyucu
	}
	if (status==2){ // Onaysiz Kart okutuldu. yeterince bekle, turn off red led + Turn on Orange led
		sleep(1);
		if (reader)
			buf[0]=68; // Ic okuyucu
		else
			buf[0]=64; // Dis okuyucu
	}
	res = write( fd, buf, 1 );

	releaselock();

}

void * listen_serial(void *arg) {		// client in server hidden
	int length;
  int res, i, card_control;
  char cn[256];
  unsigned char buf[255];
  unsigned int bit, status = READY, even_parity, odd_parity, msb, lsb, tmp_bit, card_number;
  FILE *log_file;

   int DoorID= atoi(myDoorIDstr);
   int inner = 1;
   char str[255], strtmp[255];
   int len=0;
   struct tm *ptr;
   time_t tm;
   int Card_Number=12343;
   char Card_Number_Str[9];
   paramsptr newparams;

     pthread_t pt;

		printf("\nSTATUS is READY!\n");

// main loop
 while (1) {     /* loop until we have a terminating condition */
 /* read blocks program execution until a line terminating character is
    input, even if more than 255 chars are input. If the number
    of characters read is smaller than the number of chars available,
    subsequent reads will return the remaining chars. res will be set
    to the actual number of characters actually read */
//    buf[0]=10;
//res=0;
//	printf("res:%d\n", res);
//    while ((buf[0]==10) || (buf[0]==13))  {
//    while (!res)  {
//		grablock();
    	res = read(fd,buf,1);
//	releaselock();
//	printf("res:%d\n", buf[0]);
//    	if (buf[0]==10)
//		printf("\n");
//    }
//printf("\n!!!\n");
    buf[res]=0;             /* set end of string, so we can printf */
    bit = buf[0];
    if (status == READY) {
    	status++;
	card_number=even_parity=odd_parity=0;
	bit = bit & 3;	// logical and with 11
	if (bit == 0) {		// the code 00 can not be recieved from C4 Control Card
//		if (buf[0]!=10)
			printf("ERROR:Unexpected code!\n");
		status = READY;
		printf("\nSTATUS is READY!\n");
	}
	if (bit == 2) {		// if the code is 01, it is Inner Wiegand Proximity Reader
		printf("Inner Wiegand Proximity Reader:\n");
		inner=1;

	}
	if (bit == 1){		// if the code is 10, it is Outer Wiegand Proximity Reader
		printf("Outer Wiegand Proximity Reader:\n");
		inner=0;
	}
	if (bit == 3) {	// if the code is 11, it is Door Status Sensor
		printf("Door Status Sensor:\n");
		bit = buf[0];
		printf("%d\n", bit);
		bit = bit & 4;
   strcpy(str, "");
     strcat(str, "0");
		if (bit != 4){	// if the third bit is 1, the door is open
			printf("Door is opened.\n");
   strcat(str, DOOR_OPENED_NORMALLY);
			}
		else {		// otherwise closed.
			printf("Door is closed.\n");
   strcat(str, DOOR_CLOSED_NORMALLY);
}
// Log Data Paketleme
//   strcat(str, DOOR_STAY_OPENED);
//   strcat(str, DOOR_OPENED_BY_FORCE);
   tm = time(NULL);
   ptr = localtime(&tm);
   strftime(strtmp ,100 , "%d%m%Y%H%M%S",ptr);
   strcat(str,strtmp);
   printf("date:%s\n", strtmp);
   len= sprintf(Card_Number_Str, "%d", Card_Number);
   strncpy(strtmp, zeros, (8 - len));
   strtmp[8-len]='\0';
   strcat(str,strtmp);
   strcat(str,Card_Number_Str);
   len= PacketData(DoorID, LogData, 23, str);
   printf("packet:%s, length:%d \n", packet, len);  //MainBoarddan Servere gnderilecek 'packet'

//		status = READY;
		status=SIXTH_READ;
//		printf("\nSTATUS is READY!\n");
// writing log file
log_file=fopen("logs", "a");
if (log_file==NULL)
	printf("file error\n");
fprintf(log_file, "%s\n", packet);
fclose(log_file);

	}
    }
    else
    	status++;
    if (status==FIRST_READ) {
//printf("FIRST_READ:");
	lsb = bit = buf[0];
	lsb = lsb >> 2;		// taking two bit which is LSB
	lsb = lsb & 1;		// taking third bit which is MSB
	bit = bit >> 3;		// shifting one bit to right
	for (i=0; i<4; i++) {	// reading 4 bit by shifting
		tmp_bit = bit & 1;
		even_parity = even_parity + tmp_bit;
		card_number=card_number+tmp_bit*pow(2,i);
		bit = bit >> 1;
	}
	tmp_bit = bit & 1;
	if (tmp_bit)
		printf("1.H A T A ! ! !\n");
    }
    if (status==SECOND_READ) {
//printf("SECOND_READ:");
	bit = buf[0];
//	bit = bit >> 1;
	for (i=0; i<7; i++) {
		tmp_bit = bit & 1;
		clientptr me = (clientptr) arg;
	int clientnum = me->clientnum;even_parity = even_parity + tmp_bit;
		card_number=card_number+tmp_bit*pow(2,(i+4));
		bit = bit >> 1;
	}
	tmp_bit = bit & 1;
	if (tmp_bit)
		printf("2.H A T A ! ! !\n");
    }
    if (status==THIRD_READ) {
//printf("THIRD_READ:");
	bit = buf[0];
//	bit = bit >> 1;
	tmp_bit= bit & 1;
	even_parity = even_parity + tmp_bit;
	even_parity = even_parity & 1;
printf("even parity:%d, lsb:%d ", even_parity, lsb);
	card_number=card_number+tmp_bit*pow(2,11);
	bit = bit >> 1;
	for (i=0; i<6; i++) {
		tmp_bit = bit & 1;
		odd_parity = odd_parity + tmp_bit;
		card_number=card_number+tmp_bit*pow(2,(i+12));
		bit = bit >> 1;
	}
	if (lsb==even_parity)
		printf("even parity is verified!\n");
	else
		printf("even parity is NOT verified!\n");
	tmp_bit = bit & 1;
	if (tmp_bit)
		printf("3.H A T A ! ! !\n");
    }

    if (status==FOURTH_READ) {
//printf("FOURTH_READ:");
	bit = buf[0];
	for (i=0; i<6; i++) {
		tmp_bit = bit & 1;
		odd_parity = odd_parity + tmp_bit;
		card_number=card_number+tmp_bit*pow(2,i+18);
		bit = bit >> 1;
	}
//	bit = bit >> 1;
	msb = bit & 1;
	bit = bit >> 1;
	odd_parity++;
	odd_parity = odd_parity & 1;
printf("odd parity:%d, msb:%d ", odd_parity, msb);
	if (msb==odd_parity)
		printf("odd parity is verified!\n");
	else
		printf("odd parity is NOT verified!\n");
	tmp_bit = bit & 1;
	if (tmp_bit)
		printf("4.H A T A ! ! !\n");
//	status = READY;
	printf("card number:%d ", card_number);
// Send the read character
//    rc = send(sd, buf, 2, 0);
//	printf("card number2:%s:\n", i2s(card_number));
//    str_tmp=i2s(card_number);	// convert integer card number to string and put on str_tmp variable!
    length= sprintf(cn, "%d", card_number);
/*xxx*/
// Log Data Paketleme
   strcpy(str, "");
   if (inner)
     strcat(str, "1");
   else
     strcat(str, "0");
	for (card_control=0; card_control<aindex;card_control++)
		if (cn_array[card_control]==card_number)
			break;
newparams = malloc(sizeof(params));
//	grablock();
	if (cn_array[card_control]==card_number) {
   strcat(str, ACCESS_GRANTED);
buf[1]=0;
if (inner) {
	grablock();
	buf[0]=12;
	res = write( fd, buf, 1 );
	newparams->status=1;
	newparams->reader=1;
   	releaselock();
	pthread_create(&pt,NULL,control_lock_led_buzer,(void *)newparams);
} else {
	grablock();
	buf[0]=8;
	res = write( fd, buf, 1 );
	newparams->status=1;
	newparams->reader=0;
   	releaselock();
	pthread_create(&pt,NULL,control_lock_led_buzer,(void *)newparams);
}


	    }
	else {
buf[1]=0;
if (inner) 
{
	if (InnerReaderActive==1)
	{
		grablock();
		buf[0]=36;
		res = write( fd, buf, 1 );
		newparams->status=2;
		newparams->reader=1;
		releaselock();
		pthread_create(&pt,NULL,control_lock_led_buzer,(void *)newparams);
	}
} 
else 
{
	if (OuterReaderActive==1)
	{
		grablock();
		buf[0]=32;
		res = write( fd, buf, 1 );
		newparams->status=2;
		newparams->reader=0;
		releaselock();
		pthread_create(&pt,NULL,control_lock_led_buzer,(void *)newparams);
	}
}

//pthread_create(&pt,NULL,listen_serial,(void *)ld);  control_lock_led_buzer(int status, int reader)
   strcat(str, ACCESS_DENIED); }
   sleep(1); //xxx
//   	releaselock();

//   strcat(str, DOOR_OPENED_NORMALLY);
//   strcat(str, DOOR_CLOSED_NORMALLY);
//   strcat(str, DOOR_STAY_OPENED);
//   strcat(str, DOOR_OPENED_BY_FORCE);
   tm = time(NULL);
   ptr = localtime(&tm);
   strftime(strtmp ,100 , "%d%m%Y%H%M%S",ptr);
   strcat(str,strtmp);
   printf("date:%s\n", strtmp);
   len= sprintf(Card_Number_Str, "%d", card_number);
   strncpy(strtmp, zeros, (8 - len));
   strtmp[8-len]='\0';
   strcat(str,strtmp);
   strcat(str,Card_Number_Str);
   printf("Card_Number_Str:%s \n", Card_Number_Str);  //MainBoarddan Servere gnderilecek 'packet'
   len= PacketData(DoorID, LogData, 23, str);
   printf("packet:%s, length:%d \n", packet, len);  //MainBoarddan Servere gnderilecek 'packet'
// writing log file
if (((inner) && (InnerReaderActive==1)) || ((!inner) && (OuterReaderActive==1)))
{
	log_file=fopen("logs", "a");
	if (log_file==NULL)
		printf("file error\n");
	fprintf(log_file, "%s\n", packet);
	fclose(log_file);
	strcat(packet,"             ");
	strcpy(log_file_sent_array[sacounter++], packet);
}
/* burada gonderim yapiliyor...
    rc = send(sd, cn, length+1, 0);
    if(rc<0) {
      perror("ERROR:cannot send data!");
      close(sd);
      exit(1);
    }
    else
	printf("Data has been sent:%s (length:%d)\n", cn, length);
    }
*/
}
    if (status==FIFTH_READ) {
	status = READY;
printf("\nSTATUS is READY!\n");
	}
    if (status==SIXTH_READ) {
	status = FOURTH_READ;
	}
//bit = buf[0];
//    printf("Onaltilik:%X\n",bit);
//    printf("%s",buf);

//    if (buf[0]=='z') STOP=TRUE;

}
/* restore the old port settings */
 tcsetattr(fd,TCSANOW,&oldtio);

}

int PacketData(int did, const char * code, int datalen, char * data)
{
   char str[255], strtmp[255];
   int len=0;
   len= sprintf(strtmp, "%d", did);
   strncpy(str, zeros, (10 - len));
   str[10-len]='\0';

   strcat(str, strtmp);
   strcat(str, code);
   strcat(str, data);

   strcpy(packet, str);
   return(strlen(str));
}

void *handle_client(void *arg)
{
	clientptr me = (clientptr) arg;
	int clientnum = me->clientnum;
	int sd = me->sd;
	char buf[BUFSIZE], str[BUFSIZE], strtmp[BUFSIZE];
	int n, cnt=0, beginning=1;
	char Card_Number_Str[9], DoorID_Str[11], msg_type[4], reader[2], msg_no[3], date[17], timeOfDate[17],
	     Int_Number[9], Attendance[2];
	char event[9], threshold[9],two[2], inner[2], outer[2],stay[9],lock[2],sensor[2],calendar[9],online[2];

	char Time[6], Calendar_No_Str[9], Action_No_Str[9], Day[2], Type[2];

	FILE *log_file, *log_file_ack, *card_file, *door_file, *calendar_file,*log_sent;
	int res;
	int saloop;
	
	time_t curtime;
	struct tm *loctime;	
	char TOD[17], D[17], Da[2];
	int ekle;
	int IsGetRecord=0;
	free(arg);
	char myDateStr[255], strtmp1[255];

//  By default a new thread is joinable, we don't
//  really want this (unless we do something special we
//  end up with the thread equivalent of zombies). So
//  we explicitly change the thread type to detached
	pthread_detach(pthread_self());

	printf("Thread %d started for client number %d (sd %d)\n",
	pthread_self(),clientnum,sd);

	while ((read(sd,buf,51))>2)
	{
		strncpy(DoorID_Str, buf, 10);
		DoorID_Str[10]= '\0';
		printf("Door ID:%s\n", DoorID_Str);

		strncpy(msg_type, buf+10, 3);
		msg_type[3]= '\0';
		printf("Message Type:%s\n", msg_type);

		if (!strcmp(myDoorIDstr, DoorID_Str))
		{
			IsGetRecord=1;
			if (atoi(msg_type)==0)
			{
				if (beginning==1) 
				{
					log_file_ack=fopen("logs_ack", "a+");
					if (log_file_ack==NULL)
						printf("file error\n");
				}
			}
			else if (atoi(msg_type)==101)
			{
			
			}
			else if (atoi(msg_type)==110)
			{
				if (beginning==1) 
					if (cnt==0)
					{
						door_file=fopen("doors", "w");
						if (door_file==NULL)
							printf("file error\n");
					} 
					else if (cnt==1)
					{
						card_file=fopen("cards", "w");
						if (card_file==NULL)
							printf("file error\n");
					}
					else if (cnt==2)
					{
						calendar_file=fopen("calendars", "w");
						if (calendar_file==NULL)
							printf("file error\n");
					}
			}	
			if (cnt==0) printf("DOOR\n");
			if (cnt==1) printf("CARD\n");
			if (cnt==2) printf("CALE\n");
			printf("%s\n",buf);
			strncpy(DoorID_Str, buf, 10);
			DoorID_Str[10]= '\0';
			printf("Door ID:%s\n", DoorID_Str);

			if (!strcmp(myDoorIDstr, DoorID_Str))
			{
			strncpy(msg_type, buf+10, 3);
			msg_type[3]= '\0';
			printf("Message Type:%s\n", msg_type);

			if (atoi(msg_type)==111)
			{
				strcpy(myDateStr, "date ");
				strncpy(myDateStr+5, buf+18, 2);
				strncpy(myDateStr+7, buf+16, 2);
				strncpy(myDateStr+9, buf+24, 4);
				strncpy(myDateStr+13, buf+20, 4);
				strcpy(myDateStr+17, ".");
				strncpy(myDateStr+18, buf+28, 2);
				strcpy(myDateStr+20, "\n");
				printf("%s\n", myDateStr);
				if (system(myDateStr)==-1) printf ("Error updating date to:%s\n",myDateStr);
				else printf ("Date successfully updated to:%s\n",myDateStr);
			}
			else			
			if (atoi(msg_type)==0)
			{
				grablock();
				
				if (beginning)
				{
					//log_file_ack=fopen("logs_ack", "a+");
					//if (log_file_ack==NULL)
					//	printf("file error\n");
					beginning= 0;
				}
				cnt= 3;

				buf[10]= '0';
				buf[11]= '0';
				buf[12]= '1';
				buf[51]= '\0';
				buf[38]= '\0';
				
				printf("AAAAA:%s*****%d*****\n", buf, strlen(buf));
				for (saloop=0; saloop<sacounter; saloop++)
				{
					strcpy(strtmp1,log_file_sent_array[saloop]); strtmp1[38]= '\0';
					printf("BBBBB:%s*****%d*****\n", strtmp1, strlen(strtmp1));
					if (!strcmp(strtmp1, buf))
					{
						strcpy(log_file_sent_array[saloop], log_file_sent_array[sacounter-1]);
						strcpy(log_file_sent_array[sacounter-1], "");
						sacounter--;
						printf("Acked - record deleted from sent array\n");
						break;
					}
				}
				if (sacounter==0)
				{
					log_sent=fopen("logs_sent", "w");
					if (log_sent==NULL)
						printf("file error\n");
					fclose(log_sent);
				}
				//if (sacounter==0) myStartedUp= 0;
				buf[38]= '\0';
				fprintf(log_file_ack, "%s\n", buf);
//				fclose(log_file_ack);
				releaselock();
			}
			else

			if (atoi(msg_type)==101)
			{
				grablock();
				printf("OK222222 \n");
			// reading log file
				log_file=fopen("logs", "r");
				if (log_file==NULL)
					printf("file error\n");
				while (fscanf(log_file, "%s\n", &str)!=EOF)
				{
					printf("%s\n",str);
					n = write(sd,str,strlen(str));
					if (n<0) {printf("Error writing to client.\n");}
				}
				fclose(log_file);
				log_file=fopen("logs", "w");
				if (log_file==NULL)
					printf("file error\n");
				fclose(log_file);
				releaselock();
			}
			else
				if (atoi(msg_type)==110)
				{
					printf("RelDoDB\n");
					if (cnt==0)
					{ //  DOOR bilgisini parala dosyaya yaz
						if (beginning)
						{
							//door_file=fopen("doors", "w");
							//if (door_file==NULL)
							//	printf("file error\n");
							beginning= 0;
						}
//Door Data Paket Ama////////////////////////////////////////////////////////////////////////////
						strncpy(event, buf+13, 8);
						event[8]='\0';
						printf("Event_Capacity:%s\n",event);

						strncpy(threshold, buf+21, 8);
						threshold[8]='\0';
						printf("Threshold_Capacity:%s\n",threshold);

						strncpy(two, buf+29, 1);
						two[1]='\0';
						printf("Two_Readers_Present:%s\n",two);

						strncpy(inner, buf+30, 1);
						inner[1]='\0';
						printf("Inner_Reader_Active:%s\n",inner);
						InnerReaderActive= atoi(inner);

						strncpy(outer, buf+31, 1);
						outer[1]='\0';
						printf("Outer_Reader_Active:%s\n",outer);
						OuterReaderActive= atoi(outer);
						
						strncpy(stay, buf+32, 8);
						stay[8]='\0';
						printf("Stay_Open_Duration:%s\n",stay);

						strncpy(lock, buf+40, 1);
						lock[1]='\0';
						printf("Door_Locked:%s\n",lock);

						strncpy(sensor, buf+41, 1);
						sensor[1]='\0';
						printf("Door_Forced_Sensor:%s\n",sensor);

						strncpy(calendar, buf+42, 8);
						calendar[8]='\0';
						printf("Used_Calendar:%s\n",calendar);

						strncpy(online, buf+50, 1);
						online[1]='\0';
						printf("Online_Log:%s\n",online);

						fprintf(door_file, "%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\n",
							DoorID_Str, msg_type, event, threshold, two, inner,
							outer, stay, lock, sensor, calendar, online);
//Door Data Paket Ama Sonu////////////////////////////////////////////////////////////////////////
						OpenStayDuration= atoi(stay);
						
						myEventCapacity=atoi(event) ; myThresholdCapacity=atoi(threshold) ; myOnline=atoi(online);
						
						if (lock[0]==48)
						{
							buf[0]=12;
							res = write( fd, buf, 1 );  // Open the Door + Green Led of inner reader
							buf[0]=8;
							res = write( fd, buf, 1 );  // Open the Door + Green Led of outer reader
						}
						else
						{
							buf[0]=20;
							res = write( fd, buf, 1 );  // Lock the Door + Orange Led of inner reader
							buf[0]=16;
							res = write( fd, buf, 1 );  // Lock the Door + Orange Led of outer reader
						}
						//res = write( fd, "\0", 1 );
					}
					if (cnt==1)
					{ // CARD bilgisini parala dosyaya yaz
						if (beginning)
						{
							//card_file=fopen("cards", "w");
							//if (card_file==NULL)
							//	printf("file error\n");
							beginning= 0;
							aindex=0;
						}
//Card Data Paket Ama////////////////////////////////////////////////////////////////////////////
						strncpy(Card_Number_Str, buf+13, 8);
						Card_Number_Str[8]='\0';
						printf("Card_Number:%s\n",Card_Number_Str);

						strncpy(Int_Number, buf+21, 8);
						Int_Number[8]='\0';
						printf("Internal_Number:%s\n",Int_Number);

						strncpy(Attendance, buf+29, 1);
						Attendance[1]='\0';
						printf("Attendance_Follow:%s\n",Attendance);

						strcpy(date, "");

						strncpy(strtmp, buf+30, 2);
						strtmp[2]='\0';
						strcat(date,strtmp);

						strcat(date,"-");

						strncpy(strtmp, buf+32, 2);
						strtmp[2]='\0';
						strcat(date,strtmp);

						strcat(date,"-");

						strncpy(strtmp, buf+34, 4);
						strtmp[4]='\0';
						strcat(date,strtmp);

						strcat(date," ");

						strncpy(strtmp, buf+38, 2);
						strtmp[2]='\0';
						strcat(date,strtmp);

						strcat(date,":");

						strncpy(strtmp, buf+40, 2);
						strtmp[2]='\0';
						strcat(date,strtmp);
						strcat(date,":00");

						printf("date:%s\n",date);
//Card Data Paket Ama Sonu////////////////////////////////////////////////////////////////////////
						fprintf(card_file, "%s\t%s\t%s\t%s\t%s\t%s\n",
							DoorID_Str, msg_type, Card_Number_Str, Int_Number, Attendance, date);
						cn_array[aindex]=atoi(Card_Number_Str);
						// ///////  12-06-2005 00:00:00
						//strcpy((CardExp_Array[aindex].date), "date ");
						strncpy(CardExp_Array[aindex].date, date+6, 4);
						strncpy((CardExp_Array[aindex].date)+4,"-", 1);
						strncpy((CardExp_Array[aindex].date)+5, date+3, 2);
						strncpy((CardExp_Array[aindex].date)+7,"-", 1);
						strncpy((CardExp_Array[aindex].date)+8, date, 2);
						strcpy((CardExp_Array[aindex].date)+10, "\0");
						printf("%s\n", (CardExp_Array[aindex].date));
						
						// ///////
						strncpy(CardExp_Array[aindex].timeOfDate, date+11, 5);
						strcpy((CardExp_Array[aindex].timeOfDate)+5, "\0");
						//strncpy(CardExp_Array[aindex].date, date, 10);
						aindex++;
					}
					if (cnt==2)
					{ // CALENDAR bilgisini parala dosyaya yaz
//Calendar Data Paket Ama////////////////////////////////////////////////////////////////////////////
						if (beginning)
						{
							//calendar_file=fopen("calendars", "w");
							//if (calendar_file==NULL)
							//	printf("file error\n");
							beginning= 0;
							Calendar_Amount=0;
						}
						strncpy(Calendar_No_Str, buf+13, 8);
						Calendar_No_Str[8]='\0';
						printf("Calendar_No:%s\n",Calendar_No_Str);

						strncpy(Day, buf+21, 1);
						Day[1]='\0';
						printf("Day:%s\n",Day);

						strcpy(timeOfDate, "");

						strncpy(strtmp, buf+22, 2);
						strtmp[2]='\0';
						strcat(timeOfDate,strtmp);

						strcat(timeOfDate,":");

						strncpy(strtmp, buf+24, 2);
						strtmp[2]='\0';
						strcat(timeOfDate,strtmp);

						printf("Open_Time:%s\n",timeOfDate);

						strncpy(Action_No_Str, buf+26, 8);
						Action_No_Str[8]='\0';
						printf("Action_No:%s\n",Action_No_Str);

						strncpy(Type, buf+34, 1);
						Type[1]='\0';
						printf("Type:%s\n",Type);

						strcpy(date, "");

						strncpy(strtmp, buf+35, 2);
						strtmp[2]='\0';
						strcat(date,strtmp);

						strcat(date,"-");

						strncpy(strtmp, buf+37, 2);
						strtmp[2]='\0';
						strcat(date,strtmp);

						strcat(date,"-");

						strncpy(strtmp, buf+39, 4);
						strtmp[4]='\0';
						strcat(date,strtmp);

						printf("date:%s\n",date);
		if (atoi(Type)==1)
		{
			curtime = time (NULL);
			loctime = localtime (&curtime);
			strftime (TOD, 17, "%H:%M", loctime); //Hour:Minute
			strftime (D, 17, "%d-%m-%Y", loctime); //Day-month-Year
			strftime (Da, 2, "%u", loctime); //weekday
			if (atoi(Da) <= atoi(Day)) 
				ekle= atoi(Day)-atoi(Da);
			else
				ekle=7-(atoi(Da)-atoi(Day));
			curtime= curtime+ekle*86400;
			loctime = localtime (&curtime);
			strftime (date, 17, "%d-%m-%Y", loctime); //Day-month-Year
		}		
//Calendar Data Paket Ama Sonu///////////////////////////////////////////////////////////////////////
						fprintf(calendar_file, "%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\n",
							DoorID_Str, msg_type, Calendar_No_Str, Day,
							timeOfDate, Action_No_Str, Type, date);
							strcpy(Calendar_Array[Calendar_Amount].timeOfDate, timeOfDate);
							strcpy(Calendar_Array[Calendar_Amount].date, date);
							strcpy(Calendar_Array[Calendar_Amount].Day, Day);
							strcpy(Calendar_Array[Calendar_Amount].Action_No_Str, Action_No_Str);
							strcpy(Calendar_Array[Calendar_Amount].Type, Type);
							Calendar_Amount++;
					}
				}
				else printf("Unknown Error: #222222, Kernel Panic, Do not Panic, Please Call the Service %s\n", buf);
		}
		}
		if (strcmp(buf, "quit"))
		{
			IsGetRecord=1;
		}
		else
		{printf("***QUIT***\n");
			if (cnt==0)
			{
				if (IsGetRecord==0)
				{ 
					door_file=fopen("doors", "w");
					if (door_file==NULL)
						printf("file error\n");
					beginning= 0;
				} 
				if (beginning!=1) 
				fclose(door_file);
				beginning=1;
			}
			if (cnt==1)
			{
				if (IsGetRecord==0)
				{
					card_file=fopen("cards", "w");
					if (card_file==NULL)
						printf("file error\n");
					beginning= 0;
				}
				if (beginning!=1) 
				fclose(card_file);
				beginning=1;
			}
			if (cnt==2)
			{
				if (IsGetRecord==0)
				{
					calendar_file=fopen("calendars", "w");
					if (calendar_file==NULL)
						printf("file error\n");
					beginning= 0;
				}
				if (beginning!=1) 
				fclose(calendar_file);
				beginning=1;
			}
			if (cnt==3)
			{
				if (beginning!=1) 
				fclose(log_file_ack);
				beginning=1;
			}
			cnt++;
			IsGetRecord=0;
		}
	}

/* handle EOF */
	if (n==0)
	{
		grablock();
		close(sd);
		clients[clientnum]=0;
		printf("Client %d has left the building\n",clientnum);
		releaselock();
	}
	else
	{
		printf("Error reading from client %d - quitting\n",clientnum);
		exit(1);
	}
	printf("\n\n");
	return(NULL);
}

// The message we send to new clients if we are full
char *fullmsg = "Sorry - no vacancy\n";

//  The main sets up a passive socket and accepts new connections.  For
//  each new connection:
//    - if the server is full a message is sent to
//      the client and the client connection is terminated.
//    - otherwise a thread is started to handle the client

void * handle_calendar(void *arg)
{
	time_t curtime;
	struct tm *loctime;
	time_t next;
	struct tm *nextloctime;
	int countercal;
	char buffer[256];
	char TOD[17], D[17], Da[2], ANS[9], T[2];
	char TOD1[17], D1[17], Da1[2], ANS1[9], T1[2];
	int res; char buf[BUFSIZE];
	int card_control;
	char D2[17];
	
	printf("handle_calendar\n");
	do
	{	
		curtime = time (NULL);
		loctime = localtime (&curtime);
		//strftime (buffer, 256, "%d-%m-%Y %H:%M:%S %w", loctime); //Day-month-Year Hour:Minute:Second weekday
		strftime (TOD, 17, "%H:%M", loctime); //Hour:Minute
		strftime (D, 17, "%d-%m-%Y", loctime); //Day-month-Year
		strftime (Da, 2, "%u", loctime); //weekday	

		//printf("The current time: %s\t%s\t%s\n", TOD, D, Da);
		
		//printf("Calendar_Amount:%d\n", Calendar_Amount);
// Expiration date control
	strftime (D2, 17, "%Y-%m-%d", loctime); //Year-month-Day
	for (card_control=0; card_control<aindex;card_control++)
		if (strcmp(CardExp_Array[card_control].date, "0000-00-00"))
			if (strcmp(CardExp_Array[card_control].date, D2) <=0)
				if (strcmp(CardExp_Array[card_control].timeOfDate, TOD)<=0)
				{
					printf("Card Expired: %d\t%s\t%s\n", cn_array[card_control], 
						CardExp_Array[card_control].timeOfDate, CardExp_Array[card_control].date);
					cn_array[card_control]= cn_array[aindex];
					strcpy(CardExp_Array[card_control].timeOfDate, CardExp_Array[aindex].timeOfDate);
					strcpy(CardExp_Array[card_control].date, CardExp_Array[aindex].date);
					aindex--;
					card_control--;
				}
// End Of Expiration date control		
		for (countercal=0; countercal<Calendar_Amount; countercal++)
		{
			//printf("%s\t%s\t%s\t%s\t%s\n",
			//	Calendar_Array[countercal].timeOfDate,
			//	Calendar_Array[countercal].date,
			//	Calendar_Array[countercal].Day,
			//	Calendar_Array[countercal].Action_No_Str,
			//	Calendar_Array[countercal].Type);
			if (!strcmp(TOD, Calendar_Array[countercal].timeOfDate) && 
				    !strcmp(D, Calendar_Array[countercal].date) &&
				    !strcmp(Da, Calendar_Array[countercal].Day))
			{
				printf("Doing Calendar action.... %s\n", Calendar_Array[countercal].Action_No_Str);
				switch (atoi(Calendar_Array[countercal].Action_No_Str))
				{
					case 1://Unlock the door
						buf[0]=12;
						res = write( fd, buf, 1 );  // Open the Door + Green Led of inner reader
						buf[0]=8;
						res = write( fd, buf, 1 );  // Open the Door + Green Led of outer reader
					break;					
					case 2://Lock the door
						buf[0]=20;
						res = write( fd, buf, 1 );  // Lock the Door + Orange Led of inner reader
						buf[0]=16;
						res = write( fd, buf, 1 );  // Lock the Door + Orange Led of outer reader	
					break;					
					case 3://Inner Reader Active
						InnerReaderActive=1;
					break;					
					case 4://Inner Reader Passive
						InnerReaderActive=0;
					break;					
					case 5://Outer Reader Active
						OuterReaderActive=1;
					break;
					case 6://Outer Reader Passive
						OuterReaderActive=0;
					break;					
				}
				switch (atoi(Calendar_Array[countercal].Type))
				{
					case 1:
						next= time (NULL);
						next= next+ 604800;
						nextloctime= localtime (&next);						
						//strftime (buffer, 256, "%d-%m-%Y %H:%M:%S %w", nextloctime);
						strftime (TOD1, 17, "%H:%M", loctime); //Hour:Minute
						strftime (D1, 17, "%d-%m-%Y", loctime); //Day-month-Year
						strftime (Da1, 2, "%u", loctime); //weekday							
						strcpy(Calendar_Array[countercal].timeOfDate,TOD1);
						strcpy(Calendar_Array[countercal].date,D1);
						strcpy(Calendar_Array[countercal].Day,Da1);
					break;
					case 2:
						strcpy(Calendar_Array[countercal].timeOfDate,Calendar_Array[Calendar_Amount-1].timeOfDate);
						strcpy(Calendar_Array[countercal].date,Calendar_Array[Calendar_Amount-1].date);
						strcpy(Calendar_Array[countercal].Day,Calendar_Array[Calendar_Amount-1].Day);
						strcpy(Calendar_Array[countercal].Action_No_Str,Calendar_Array[Calendar_Amount-1].Action_No_Str);
						strcpy(Calendar_Array[countercal].Type,Calendar_Array[Calendar_Amount-1].Type);
						Calendar_Amount--;
						countercal--;
					break;
				}
			}
		}
	}while(TRUE);
}

void * handle_db(void *arg)
{
	printf("handle_db\n");
}

void * send_to_serial(void *arg)
{	// master computer'da gerek yok...
	printf("send_to_serial\n");
}

void * listen_net(void *arg)
{	// master computer'da bu isi main() yapiyor...
	printf("listen_net\n");
}

void * send_to_net(void *arg)
{	// client in server hidden
	FILE *log_file, *log_file_ack, *log_file_sent;
	int n, sock_fd, numbytes, valid=TRUE, think=0, debug_mode=FALSE, port=0;
	char buf[MAX_MSG], str[MAX_MSG], strtmp[MAX_MSG];
	struct hostent *he;
	struct sockaddr_in their_addr; // connector's address information
	int saloop;

	while(1)
	{
		if ((sacounter>=myThresholdCapacity) || ((myOnline==1) && (sacounter>0)) || (myStartedUp==1))
		{
			//myStartedUp= 0;
			grablock();
			// reading file 
			printf("send_to_net->sacounter=%d\t\tmyStartedUp=%d\n", sacounter,myStartedUp);
			for (saloop=0; saloop<sacounter; saloop++)
			{
				log_file_sent_array[saloop][38]= '\0';
				printf("Bu var:%s\n",log_file_sent_array[saloop]);
			}
			printf("BITTI\n");
					log_file=fopen("logs", "a+");
					if (log_file==NULL)
						printf("file error\n");
					for (saloop=0; saloop<sacounter; saloop++)
					{
						log_file_sent_array[saloop][38]= '\0';
						fprintf(log_file, "%s\n", log_file_sent_array[saloop]);
						printf("%s\n",log_file_sent_array[saloop]);
					}
					sacounter= 0;
					fclose(log_file);
					log_file=fopen("logs_sent", "w");
					if (log_file==NULL)
						printf("file error\n");
					fclose(log_file);
					//releaselock();
	
			sleep(2);
			printf("Trying to connect master computer...\n");
	
			if ((he=gethostbyname(SERVER_NAME)) == NULL)
			{  // get the host info
				perror("gethostbyname");
				//exit(1);
			}
			//else printf("gethostbyname:OK\n");
	
			if ((sock_fd = socket(AF_INET, SOCK_STREAM, 0)) == -1)
			{
				perror("socket");
				//exit(1);
			}
			//else printf("socket:OK\n");
	
			their_addr.sin_family = AF_INET;    // host byte order
			their_addr.sin_port = htons(SERVER_PORT);  // short, network byte order
			their_addr.sin_addr = *((struct in_addr *)he->h_addr);
			memset(&(their_addr.sin_zero), '\0', 8);  // zero the rest of the struct
			if (connect(sock_fd, (struct sockaddr *)&their_addr, sizeof(struct sockaddr)) == -1)
			{
				perror("connect");
				//exit(1);
			}
			else
			{
				//printf("connect:OK\n");
				// reading log file
				//grablock();
				
				log_file_ack=fopen("logs_ack", "a");
				if (log_file_ack==NULL)
					printf("file error\n");
	
				log_file_sent=fopen("logs_sent", "a");
				if (log_file_sent==NULL)
					printf("file error\n");
	
				log_file=fopen("logs", "r");
				if (log_file==NULL)
					printf("file error\n");
				else
				while (fscanf(log_file, "%s\n", &str)!=EOF)
				{
					fprintf(log_file_sent, "%s\n", str);
					strcat(str,"             ");
					strcpy(log_file_sent_array[sacounter++], str);
	//				printf("%s\n", log_file_sent_array[sacounter-1]);
					n = write(sock_fd,str,strlen(str));
					if (n<0)
					printf("Error writing to client %d\n");
					printf("Data has been sent:%s\n",str);
				}
				
				fclose(log_file); fclose(log_file_ack); fclose(log_file_sent);
					log_file=fopen("logs", "w");
					if (log_file==NULL)
						printf("file error\n");
					fclose(log_file);
				n = write(sock_fd,"quit\0",strlen("quit")+1);
				if (n<0)
					printf("Error writing to client %d\n");
				printf("Data has been sent:quit\n");
	
				if (close(sock_fd))
					printf("denme\n");
				//releaselock();
			}
			releaselock();
			if (myStartedUp==1) 
			  sleep(10);
			myStartedUp= 0;
			
			if (sacounter>=myEventCapacity)
			{
				InnerReaderActive= 0;
				OuterReaderActive= 0;
			}
			else
			{
				card_file=fopen("doors", "r");
				if (card_file==NULL)
					printf("file error\n");
				fscanf(card_file, "%s\t%s\t%d\t%d\t%s\t%d\t%d\t%d\t%s\t%s\t%s\t%d\n",
						strtmp, strtmp, &myEventCapacity, &myThresholdCapacity, strtmp, &InnerReaderActive,
						&OuterReaderActive, &OpenStayDuration, strtmp, strtmp, strtmp, &myOnline);
				fclose(card_file);
			}
		}
	}
}

void ReadAllConstants(void)
{
	card_file=fopen("consts", "r");
	if (card_file==NULL)
		printf("file error\n");
	fscanf(card_file, "%d\n", &SERVER_PORT);
	printf("SERVER_PORT:%d\n", SERVER_PORT);
	fscanf(card_file, "%s\n", &SERVER_NAME);
	printf("SERVER_NAME:%s\n", SERVER_NAME);
	fscanf(card_file, "%s\n", &MODEMDEVICE);
	printf("MODEMDEVICE:%s\n", MODEMDEVICE);
	fscanf(card_file, "%s\n", &myDoorIDstr);
	printf("myDoorIDstr:%s\n", myDoorIDstr);
	fclose(card_file);
}

int main(int argc, char *argv[])
{
	int ld,sd;
	struct sockaddr_in skaddr;
	struct sockaddr_in from;
	int addrlen,length;
	int i;
	clientptr newcli;
	pthread_t pt;

	char Card_Number_Str[9], DoorID_Str[11], msg_type[4], reader[2], msg_no[3], date[17], timeOfDate[17],
	     Int_Number[9], Attendance[2];
	char event[9], threshold[9],two[2], inner[2], outer[2],stay[9],lock[2],sensor[2],calendar[9],online[2];

	char Time[6], Calendar_No_Str[9], Action_No_Str[9], Day[2], Type[2];
	struct tm *ptr;
	time_t tm;
	char str[255], strtmp[255];

//  Reading all constant values for SERVER_PORT, SERVER_NAME, MODEMDEVICE
	ReadAllConstants();
//  for serial communication
//  Open modem device for reading and writing and not as controlling tty
//  because we don't want to get killed if linenoise sends CTRL-C.

	fd = open(MODEMDEVICE, O_RDWR | O_NOCTTY );
	if (fd <0) {perror(MODEMDEVICE); exit(-1); }

	tcgetattr(fd,&oldtio); // save current serial port settings
	bzero(&newtio, sizeof(newtio)); // clear struct for new port settings


//  BAUDRATE: Set bps rate. You could also use cfsetispeed and cfsetospeed.
//  CRTSCTS : output hardware flow control (only used if the cable has
//            all necessary lines. See sect. 7 of Serial-HOWTO)
//  CS8     : 8n1 (8bit,no parity,1 stopbit)
//  CLOCAL  : local connection, no modem contol
//  CREAD   : enable receiving characters
// newtio.c_cflag = BAUDRATE | CRTSCTS | CS8 | CLOCAL | CREAD;
	newtio.c_cflag = BAUDRATE | CS8 | CLOCAL | CREAD;

//  IGNPAR  : ignore bytes with parity errors
//  ICRNL   : map CR to NL (otherwise a CR input on the other computer
//            will not terminate input)
//  otherwise make device raw (no other input processing)
//  newtio.c_iflag = IGNPAR | ICRNL;
	newtio.c_iflag = IGNPAR;

//  Raw output.
	newtio.c_oflag = 0;

//  ICANON  : enable canonical input
//  disable all echo functionality, and don't send signals to calling program
	newtio.c_lflag = ICANON;

//  initialize all control characters
//  default values can be found in /usr/include/termios.h, and are given
//  in the comments, but we don't need them here
	newtio.c_cc[VINTR]    = 0;     // Ctrl-c
	newtio.c_cc[VQUIT]    = 0;     // Ctrl-\
	newtio.c_cc[VERASE]   = 0;     // del
	newtio.c_cc[VKILL]    = 0;     // @
	newtio.c_cc[VEOF]     = 0;     // Ctrl-d
	newtio.c_cc[VTIME]    = 0;     // inter-character timer unused
	newtio.c_cc[VMIN]     = 1;     // blocking read until 1 character arrives
	newtio.c_cc[VSWTC]    = 0;     // '\0'
	newtio.c_cc[VSTART]   = 0;     // Ctrl-q
	newtio.c_cc[VSTOP]    = 0;     // Ctrl-s
	newtio.c_cc[VSUSP]    = 0;     // Ctrl-z
	newtio.c_cc[VEOL]     = 0;     // '\0'
	newtio.c_cc[VREPRINT] = 0;     // Ctrl-r
	newtio.c_cc[VDISCARD] = 0;     // Ctrl-u
	newtio.c_cc[VWERASE]  = 0;     // Ctrl-w
	newtio.c_cc[VLNEXT]   = 0;     // Ctrl-v
	newtio.c_cc[VEOL2]    = 0;     // '\0'

//  now clean the modem line and activate the settings for the port
	tcflush(fd, TCIFLUSH);
	tcsetattr(fd,TCSANOW,&newtio);

//  terminal settings done, now handle input

//  reading card file for card numbers
	card_file=fopen("cards", "r");
	if (card_file==NULL)
		printf("file error\n");

	while (fscanf(card_file, "%s\t%s\t%s\t%s\t%s\t%s\t%s\n",&DoorID_Str, &msg_type, &Card_Number_Str, &Int_Number, &Attendance, &date, &timeOfDate)!=EOF)
	{
		cn2= atoi(Card_Number_Str);
		cn_array[aindex]=cn2;
		printf("cardnumber:%d\n",cn2);
		aindex++;
	}
	fclose(card_file);

	strcpy(str,myDoorIDstr);
	strcat(str,"0011");
	strcat(str,DOOR_STARTUP);
	tm = time(NULL);
	ptr = localtime(&tm);
	strftime(strtmp ,100 , "%d%m%Y%H%M%S",ptr);
	strcat(str,strtmp);
	strcat(str, "00000000             ");
	printf("length of sent msg:%d\n", strlen(str));	
	strcpy(log_file_sent_array[sacounter++], str);

	card_file=fopen("logs_sent", "r");
	if (card_file==NULL)
		printf("file error\n");
	while (fscanf(card_file, "%s\n",&log_file_sent_array[sacounter])!=EOF) {sacounter++;}
	fclose(card_file);
	
	calendar_file=fopen("calendars", "r");
	if (calendar_file==NULL)
		printf("file error\n");
	while (fscanf(calendar_file, "%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\n",
				&DoorID_Str, &msg_type, &Calendar_No_Str, &Day,
				&timeOfDate, &Action_No_Str, &Type, &date)!=EOF) 
	{		
		strcpy(Calendar_Array[Calendar_Amount].timeOfDate, timeOfDate);
		strcpy(Calendar_Array[Calendar_Amount].date, date);
		strcpy(Calendar_Array[Calendar_Amount].Day, Day);
		strcpy(Calendar_Array[Calendar_Amount].Action_No_Str, Action_No_Str);
		strcpy(Calendar_Array[Calendar_Amount].Type, Type);
		Calendar_Amount++;
	}	

	card_file=fopen("doors", "r");
	if (card_file==NULL)
		printf("file error\n");
	fscanf(card_file, "%s\t%s\t%d\t%d\t%s\t%d\t%d\t%d\t%s\t%s\t%s\t%d\n",
			strtmp, strtmp, &myEventCapacity, &myThresholdCapacity, strtmp, &InnerReaderActive,
			&OuterReaderActive, &OpenStayDuration, strtmp, strtmp, strtmp, &myOnline);
	fclose(card_file);
	
//  start up the threads
	if (!(argc>1))
	{
		//pthread_create(&pt,NULL,server_status,(void *)ld);
		//pthread_create(&pt,NULL,handle_db,(void *)ld);
		pthread_create(&pt,NULL,listen_serial,(void *)ld);
		//pthread_create(&pt,NULL,send_to_serial,(void *)ld);
		//pthread_create(&pt,NULL,listen_net,(void *)ld);
	}
	pthread_create(&pt,NULL,send_to_net,(void *)ld);
	pthread_create(&pt,NULL,handle_calendar,(void *)ld);

	if ((ld = socket( PF_INET, SOCK_STREAM, 0 )) < 0)
	{
		perror("Problem creating socket\n");
		exit(1);
	}

	skaddr.sin_family = AF_INET;
	skaddr.sin_addr.s_addr = htonl(INADDR_ANY);
	skaddr.sin_port = htons(SERVER_PORT);//xxx

	if (bind(ld, (struct sockaddr *) &skaddr, sizeof(skaddr))<0)
	{
		perror("Problem binding\n");
		exit(0);
	}

// find out what port we were assigned and print it out
	length = sizeof( skaddr );
	if (getsockname(ld, (struct sockaddr *) &skaddr, &length)<0)
	{
		perror("Error getsockname\n");
		exit(1);
	}
	pport=ntohs(skaddr.sin_port);
	printf("port:%d\n",pport);

	// put the socket into passive mode (waiting for connections)
	if (listen(ld,5) < 0 )
	{
		perror("Error calling listen\n");
		exit(1);
	}

	// do some initialization (just to make sure)
	for (i=0;i<MAXCLIENTS;i++)
		clients[i]=0;

	// now process incoming connections forever ...
	while (1)
	{
		printf("Ready for a connection...\n");
		addrlen=sizeof(skaddr);
		if ( (sd = accept( ld, (struct sockaddr*) &from, &addrlen)) < 0)
		{
			perror("Problem with accept call\n");
			exit(1);
		}

		printf("Got a connection - processing...\n");

		for (i=0;i<MAXCLIENTS;i++)
		{
			if (clients[i]==0) break;
		}

		if (i==MAXCLIENTS)
		{// TOO MANY CLIENTS
			write(sd,fullmsg,strlen(fullmsg));
			close(sd);
    		}
		else
		{
			grablock();
			clients[i]=1;
			// release the lock
			releaselock();
			newcli = malloc(sizeof(clientstruct));
			newcli->clientnum=i;
			newcli->sd=sd;
//  Start up a thread for the client
//  I don't really care what the thread ids are,
//  but as long as I get it I print it out.
			pthread_create(&pt,NULL,handle_client,(void *) newcli);
			printf("New client connected: %u\n",pt);
		}
	}
}
